package com.gree.mall.manager.logic.material.manage;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import com.gree.mall.manager.commonmapper.MaterialMapper;
import com.gree.mall.manager.enums.material.PartsOrderFlagEnum;
import com.gree.mall.manager.enums.material.PartsPushFlagEnum;
import com.gree.mall.manager.enums.material.PartsRefTypeEnum;
import com.gree.mall.manager.logic.common.CommonLogic;
import com.gree.mall.manager.plus.entity.WebsitPartsChangeSales;
import com.gree.mall.manager.plus.entity.WebsitPartsChangeSalesItem;
import com.gree.mall.manager.plus.entity.WebsitPartsSales;
import com.gree.mall.manager.plus.entity.WebsitPartsSalesItem;
import com.gree.mall.manager.plus.service.WebsitPartsChangeSalesItemService;
import com.gree.mall.manager.plus.service.WebsitPartsSalesItemService;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@Slf4j
@Service
@RequiredArgsConstructor
public class WebsitPartsSalesLogic {

    private final CommonLogic commonLogic;
    private final MaterialMapper materialMapper;
    private final WebsitPartsSalesItemService websitPartsSalesItemService;
    private final WebsitPartsChangeSalesItemService websitPartsChangeSalesItemService;

    public void updatePartsSalesOrder(WebsitPartsSales salesOrder) {
        String sheetId = commonLogic.generateMaterialNo(salesOrder.getCompanyWechatId(), "ZX", PartsRefTypeEnum.CHANGE_SALES.getKey(), 13);

        List<WebsitPartsSalesItem> partsSalesOrderItemList = websitPartsSalesItemService.lambdaQuery()
                .eq(WebsitPartsSalesItem::getSalesId, salesOrder.getId())
                .list();

        List<WebsitPartsChangeSalesItem> partsChangeSalesItemList = new ArrayList<>();
        BigDecimal totalAmount = BigDecimal.ZERO;
        for (WebsitPartsSalesItem partsSalesOrderItem : partsSalesOrderItemList) {
            // 大于0才转明细
            if (partsSalesOrderItem.getQty().compareTo(BigDecimal.ZERO) > 0) {
                WebsitPartsChangeSalesItem partsChangeSalesItem = new WebsitPartsChangeSalesItem();
                partsChangeSalesItem.setChangeSalesNo(sheetId)
                        .setIdentity(salesOrder.getIdentity())
                        .setWebsitId(salesOrder.getWebsitId())
                        .setPartsWebsitId(salesOrder.getPartsWebsitId())
                        .setPartsNumber(partsSalesOrderItem.getPartsNumber())
                        .setPartsName(partsSalesOrderItem.getPartsName())
                        .setMaterialGroupName(partsSalesOrderItem.getMaterialGroupName())
                        .setGoodsStockUnit(partsSalesOrderItem.getGoodsStockUnit())
                        .setSalesId(salesOrder.getId())
                        .setSalesPrice(partsSalesOrderItem.getSalesPrice())
                        .setMarketPrice(partsSalesOrderItem.getMarketPrice())
                        .setSecondPrice(partsSalesOrderItem.getSecondPrice())
                        .setQty(partsSalesOrderItem.getQty())
                        .setTotalAmount(partsSalesOrderItem.getSalesPrice().multiply(partsSalesOrderItem.getQty()));
                totalAmount = totalAmount.add(partsChangeSalesItem.getTotalAmount());
                partsChangeSalesItemList.add(partsChangeSalesItem);
            }
        }

        if (CollectionUtil.isEmpty(partsChangeSalesItemList)) {
            // 如果明细为空 变更为END=完结
            salesOrder.setPushFlag(PartsPushFlagEnum.END.getKey());
            salesOrder.updateById();
            materialMapper.updateItemPushFlag(salesOrder.getId(), PartsPushFlagEnum.END.toString());
            return;
        }

        // 变更为TRANS=转销售
        salesOrder.setPushFlag(PartsPushFlagEnum.TRANS.toString());
        salesOrder.updateById();
        materialMapper.updateItemPushFlag(salesOrder.getId(), PartsPushFlagEnum.TRANS.toString());

        WebsitPartsChangeSales partsChangeSales = new WebsitPartsChangeSales();

        partsChangeSales.setChangeSalesNo(sheetId)
                .setCompanyWechatId(salesOrder.getCompanyWechatId())
                .setCompanyWechatName(salesOrder.getCompanyWechatName())
                .setSalesId(salesOrder.getId())
                .setChangeType(PartsRefTypeEnum.CHANGE_SALES.getKey())
                .setTotalAmount(totalAmount)
                .setWebsitId(salesOrder.getWebsitId())
                .setWebsitName(salesOrder.getWebsitName())
                .setPartsWebsitId(salesOrder.getPartsWebsitId())
                .setIdentity(salesOrder.getIdentity())
                .setWorkerId(salesOrder.getWorkerId())
                .setWorkerName(salesOrder.getWorkerName())
                .setSalesEndTime(salesOrder.getEndTime())
                .setFlag(PartsOrderFlagEnum.SUBMIT.getKey())
                .setCreateBy("自动处理")
                .setCreateTime(DateUtil.date())
                .setUpdateBy("自动处理")
                .setUpdateTime(DateUtil.date())
                .setSubmitBy("自动处理")
                .setSubmitTime(DateUtil.date())
                .insert();

        websitPartsChangeSalesItemService.saveBatch(partsChangeSalesItemList);

        // 师傅新件库存减 临占新件库存加
//        List<WorkerStockDTO> workerStockDTOList = this.createWorkerStockDTO(partsChangeSales, partsChangeSalesItemList, PartsRefTypeEnum.CHANGE_SALES.toString(),
//                PartsStockPartsAttrEnum.NEW.toString(), PartsStockPartsAttrEnum.TEMP_NEW.toString());
//        partsStockLogic.handleWorkerStock(workerStockDTOList);
    }
}
